c*********************************************************************
c $Source: /long/munnich/CVSROOT/UCLA/QTCM/src/bndry.F,v $
c $Author: munnich $
c $Revision: 1.5 $
c $Date: 2000/06/27 01:01:38 $
c
c File: bndry.f
c
c Generic routines for reading multi-variable bndary data files 
c   (currently read monthly climatology only); 
c Two different types of bndary data are handled currently:
c  bndry1: type 1 data, assumed centered on the 15th of each month, 
c     linear interpolation in between; example: monthly SST
c  bndry2: type 2 data from 15th to the 15th (e.g.,00000101.var covers 
c     from dec15 to jan15th); no interpolation. example: SST tendency
c
c The date information is converted into file names:
c   In this version, bc file names include the date and variable name
c   in this way:
c               file name = yyyymmdd.var
c   where yyyy = year , mm = month, dd=dayofmonth, var = a variable name.
c   if seasonal, yyyy=0000
c   dd=15: type 1 data; dd=01: type 2 data 
c
c Code history:
c   Original version:  Ning Zeng/Bill Weibel  May 1998, adopted from ocean.f
c   Modified to interpolate Aug 1998 by K. Hales
c
c Routines called but not in this file
c   julian	       in driver_util.F
c   TimeInterp         in utilities.F
c*********************************************************************


            subroutine bndry1(var, vardir, vardummy)
c*******************************************************************
c Time evolving boundary conditions other than real-time SST:
c   prescribed fields such as observed soil moisture, surface
c   radiative fluxes, cloud cover, etc.
c******************************************************************
 
      implicit none
#include "hgrid.h"
#include "calendar.h"
#include "bndry.h"

      REAL vardummy(NX,NY)
      character*(3) var
      character*(*) vardir

c- Altered
c    vardummy

c- Local Variables and Functions
      integer julian
      integer orig_time1,orig_time2,prior_dofm
      integer month
      character*(3) var0(NVAR)
      integer i,j,var_index,m,num_var
      integer day1,day2
      REAL thistime,ndays
      logical firstcall(NVAR)

      data var0/NVAR*'   '/		!array of var names
      data firstcall/NVAR*.true./
      data prior_dofm/0/
      data num_var/0/			!variable number corres. to
 					!position in var0
      data var_index/1/

      save prior_dofm,orig_time1,orig_time2
      save firstcall
      save var_index,num_var



c *************************************************************
c Assign an index for each variable and initialze the variables
c *************************************************************

c Assign a value of 'var_index' corresponding to each var called 
c (like num_var above)
c 'var_index' is what we called 'k'
      do m=1,NVAR
         if (var.eq.var0(m)) then
             var_index=m
             firstcall(var_index)=.false.
         end if
      end do

c this loads up the array var0 with variable names, only first
c time per variable; 
      if(firstcall(var_index)) then
 	num_var=num_var+1
        if (num_var.gt.NVAR) then	
           write(*,*) 'Error: capacity of array "var0" exceeded.'
           write(*,*) 'Remedy: increase size of NVAR in bndry.h'
           stop
        end if
        var_index=num_var
        var0(var_index)=var
        call bndry1_init(var,vardir,var_index) 	!initialize the var
      end if  


c **************************************************************
c Get values for var_prior and var_next
c **************************************************************
c	Basically the structure is the same as 
c 	used for reading and interpolating sst
c
c Reset time1 and time2 if bndry is called more than once 
c in a month (ie used to get more than one variable)
      if (dateofmodel.eq.prior_dofm) then
    	time1=orig_time1
	time2=orig_time2
      end if
         
c Read in the variable of the next month 
c This is altered from subroutine ocean (time1,time2 equivalent to
c date1,date2):
      if (dateofmodel .ge. time2) then
         do j=1,NY
         do i=1,NX
           var_prior(i,j,var_index) = var_next(i,j,var_index)
         end do
         end do
	 orig_time1=time1		!keep the unadvanced time1 and
	 orig_time2=time2		!time2 for the next var called
         time1 = time2
         time2 = time1 + 100                     !advance time2 by 1 month
         month=mod(time1-15,10000)/100           !if month=12
         if(month.eq.12)time2= time1+10000-1100   !  set to Jan of next year
         call readvar(time2,var,vardir, var_next(1,1,var_index))
         prior_dofm=dateofmodel         !set prior_dateofmodel
      endif

 

c *************************************************************
c Interpolate Variable
c *************************************************************
c do interpolation
c below is modified from subroutine Ocean
c ndays (=interval) needs to be passed!

c     timeinterp requires julian dates or some other monotonic time.
      day1 = julian(time1)
      day2 = julian(time2)
c		!!!!! ndays needs to be passed somehow
      ndays=1.
      thistime = dayofmodel + ndays/2.       ! compute timemean of sst
      call TimeInterp(var_prior(1,1,var_index),var_next(1,1,var_index)
     &     ,vardummy(1,1) ,day1,day2,thistime,nx,ny)

 

      return
      end



c ***********************************************************
c Subroutine readvar: reads the variable from monthly
c climatological data files
c ****************************************************************

      subroutine readvar(date,var_name,vardir,vardummy)

      implicit none
#include "hgrid.h"
#include "calendar.h"
#include "bndry.h"
 
      REAL vardummy(NX,NY)
      character*(3) var_name
      character*(*) vardir
      integer ios
 
c- Local Variables and Functions
      integer lnblnk,month
      character*(2) monthc
      character*(12) varfile
      integer date


c assign the file name 
      month=mod(date,10000)/100
      write(monthc,'(i2.2)') month
      varfile='0000'//monthc//'15.'//var_name    !file format: 0000mm15.var
 
c read the var
      open(13,file=vardir(1:lnblnk(vardir))//'/'//varfile
     &       ,form='formatted',status='old',iostat=ios)
      if(ios.ne.0) then
         write(6,*) 'readvar: ERROR opening data file: ',
     $        vardir(1:lnblnk(vardir))//'/'//varfile
         write(6,*) 'Readvar inputs: '
         write(6,*) '   date    = ',date
         write(6,*) '   varname = ',var_name
         write(6,*) '   vardir  = ',vardir
         stop
      endif
      
      read(13,*) vardummy
      close(13)
 

      write (*,*) 'bndry: Read ',var_name,' at model date', date
     &   ,' from file'
      write(*,*) '         "',vardir(1:lnblnk(vardir))//'/'//varfile,'"'

 
      return
      end

c*********************************************************************

c*********************************************************************


      Subroutine bndry1_init(var,vardir,var_index)

c
c  Get the initial boundary data for the dates on either side of an
c  arbitrary starting date
c
c this is from subroutine oceaninit in mxlayer.F modified for bndry

      implicit none
#include "hgrid.h"
#include "bndry.h"
#include "calendar.h"

      integer i,j,var_index
      character*(3) var
      character*(*) vardir
 
      write (*,*) "BndryInit: get the initial boundary data"

c-  get the dates(centered on the 15th) on either side of the starting date.
c before the 15th
      if(day0.ge.1.and.day0.le.15) then 
        time1 = year0*10000 + (month0-1)*100 + 15
        if(month0.eq.1)time1= (year0-1)*10000 + 12*100 + 15
        time2= year0*10000 + month0*100 + 15
      end if
c after the 15th
      if(day0.ge.15.and.day0.le.31) then
        time1= year0*10000 + month0*100 + 15
        time2 = year0*10000 + (month0+1)*100 + 15
        if(month0.eq.12)time2= (year0+1)*10000 + 1*100 + 15
      end if
      

      call readvar(time1,var,vardir,var_prior(1,1,var_index))

      do j=1,NY
      do i=1,NX
        var_now(i,j) = var_prior(i,j,var_index)      !Fudge the initial 
      end do
      end do

      call readvar(time2,var,vardir,var_next(1,1,var_index))
 
      return
      end

c*********************************************************************

            subroutine bndry2(var, vardir, vardummy)
c************
c read in seasonal boundary data centered on the 1st of a month.
c NO time interpolation needed 
c for q-flux (temperature tendency term) calculation  
c*********************************
 
      implicit none
#include "hgrid.h"
#include "calendar.h"

      REAL vardummy(NX,NY)
      character*(3) var
      character*(*) vardir

c- Altered
c    vardummy

c- Local Variables and Functions
      integer lnblnk, month, day, month_prior
      integer month_read
      character*(2) monthc
      character*(12) varfile

      data month_prior/0/
      save month_prior

      month=monthofyear  ! mod(dateofmodel,10000)/100
      day=dayofmonth     ! mod(dateofmodel,10000)-month*100
      month_read=month
      if(day.ge.15) month_read=month+1  
      if(month_read.eq.13) month_read=1
      if(month_read.eq.month_prior) return

      write(monthc,'(i2.2)') month_read              !convert into a string
      varfile='0000'//monthc//'01'//'.'//var    !file format: 0000mm01.var

c read the var
      open(13,file=vardir(1:lnblnk(vardir))//'/'//varfile
     &       ,form='formatted',status='old')
      read(13,*) vardummy
      close(13)

      write (*,*) 'bndry: Read ',var,' at model date', dateofmodel
     &   ,' from file'
      write(*,*) '         "',vardir(1:lnblnk(vardir))//'/'//varfile,'"'

      month_prior=month_read

      return
      end
c*********************************************************************
c ********************************************************************
c end bndry.F
c ********************************************************************
