      Subroutine DriveInit (lday, ival)
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "calendar.h"

      integer lastday,interval
      namelist /driverdata/ title,bnddir,SSTdir,outdir ,runname
     1  ,landon,SSTmode,year0,month0,day0,lastday,interval
     2  ,noout,ntout,ntouti,mrestart,dt,viscx,viscy,viscxq
     3  ,viscyq,mt0

      integer lday,ival

c initialize the namelist
      title='QTCM default title'
      bnddir='../bnddata'
      SSTdir='../bnddata/SST_Reynolds_oi'
      outdir='../proc'
      runname='runname'     ! if runname='runname', ncout will make up the output name by date
      landon=1           !interactive land
      SSTmode='seasonal'
      year0 = 1       !must be >= 1 and <=9998 for file name conversion
      month0 = 1
      day0 = -1       ! -1 and mrestart=1: Use date saved in the restart file
      lastday = DAYSPERYEAR
      interval = 1
      noout=0
      ntout=-30        !monthly mean output
      ntouti=-30       !monthly instantaneous data output
      mrestart=1
      dt=1200.
      viscx=4.0e5
      viscy=4.0e5
      viscxq=9.0e5
      viscyq=9.0e5
      mt0=6

c user input
      open(13, file='driver.in', status='old')
      read (13,driverdata)
      close(13)
      write (*,driverdata)

      lday=lastday
      ival=interval

      return
      end


             Subroutine finish
c************
c Finish QTCM run.  Close output files.  Restart options.
c
      implicit none

c --File I/O:  The following units are closed on exit:
c     unit 21 = output file q.out
c     unit 22 = output file qmean.out
c     unit 27 = output file timeseries.out
c*****************************************************************
 
      call out_restart            !force out_restart at the end
      close(21)  
      close(22)
      close(27)
      write(*,*) 'Success !!'
 
      return
      end
 

      subroutine TimeManager(itstep)
c     
c  timemanager  -   set the model calendar
c     
c  A "date" is coded in an integer as yyyymmdd.
c  inputs:
c  it  -  integer equal to the current day of the  model integration 
c
c  Using a 360 or 365 day calendar 
c
      implicit none
      include 'driver.h'
#include "calendar.h"

      integer day, yearofmodel,itstep

c
c time of day; Note: it is assumed that it is 00Z when this is first called,
c    otherwise an offset should be  used here.

!      itsteps = itstep

!      timeofday = (itstep-1)*dt                ! seconds since start 
!      timeofday = mod(timeofday,86400)         ! seconds from 00Z
 

!      dayofmodel = (itstep-1)*dt / 86400 + 1    !dt in seconds
      dayofmodel = itstep

!      day = dayofmodel + (month0-1)*30 + day0-1  !day since Jan.1 of year0
      day = dayofmodel -1+ cummonth(month0) + day0-1 !day since 00:00ZJan.1 of year0

!      year = (day-1)/360 + year0                 !year
      yearofmodel = day/DAYSPERYEAR + year0

!      dayofyear = mod(day-1,360)+1                         !day of year
      if (SSTmode .eq. 'perpetual') then
         dayofyear=mod((day0-1)+cummonth(month0),DAYSPERYEAR)+1
      else
         dayofyear=mod((day0-1)+cummonth(month0)+itstep-1,DAYSPERYEAR)+1
      endif

!      monthofyear = (dayofyear-1)/30 + 1

      monthofyear=12
      do while(cummonth(monthofyear).ge.dayofyear)
         monthofyear = monthofyear-1
      enddo

!      dayofmonth = dayofyear - (monthofyear-1)*30
      dayofmonth = dayofyear - cummonth(monthofyear)

      dateofmodel = yearofmodel*10000 + monthofyear*100 + dayofmonth

!      write(*,*) 'TimeManager: dayofmodel     = ', dayofmodel
!      write(*,*) '             dayofyear      = ', dayofyear
!      write(*,*) '             dateofmodel    = ', dateofmodel
!      write(*,*) '             year-month-day = ',yearofmodel,
!     1           '-',monthofyear,'-',dayofmonth

      return
      end


      Subroutine set_qclock(it)
c
c Advance the internal clock of QTCM, using the model step as a reference.
c The clock is intended for components of the QTCM which have diurnal cycles.
c
c timeofday is set to the current time of day, in seconds.
c See driver.h
c
c Note: it is assumed that it is 00Z when this is first called, otherwise
c   an offset should be  used here.  NZ
      implicit none
      integer it
! for dt
#include "driver.h"
! for timeofdary
#include "calendar.h"

      timeofday = mod(int(dt)*(it-1),86400)

      return
      end


        integer function julian(date)
c
c  return the "date" as the number of days since the model reference year.
c
      implicit none
#include "calendar.h"
        integer date, day, month, year
        year   = date/10000 - year0
        month  = mod(date,10000)/100 
        day    = mod(date,100)-day0+1
        julian = day+cummonth(month)-cummonth(month0)+year*DAYSPERYEAR

!        print*,'Julian: julian= ',julian,' date = ',date,
!     1    ' year = ',year
!     1     ,' month = ',month, ' day = ', day
        return
        end
