MODULE nc_file_data

  integer:: ncid,dims(3)
  
END MODULE

SUBROUTINE nc_init(ncfnm,nlon,nlat,ismean)

  USE nc_file_data

  IMPLICIT NONE
!  INCLUDE 'netcdf.inc'
#include <netcdf.inc>
#include "driver.h"
#include "calendar.h"
 
  INTEGER, INTENT (IN)            :: nlat,nlon 
  CHARACTER (LEN=80), INTENT (INOUT) :: ncfnm
  LOGICAL, INTENT (IN)            :: ismean

  real              :: scr(nlon)
  INTEGER           :: id,iret,dim_stype(2),dim_lat,dim_lon,dim_tim,i,lnblnk  !,ncid,dims(3)
  CHARACTER (LEN=8) :: date
  CHARACTER (LEN=10) :: time
  CHARACTER (LEN=21) :: tstring
  LOGICAL           :: exans
!
  iret=0
  if (runname(1:7).eq.'runname') then
     CALL date_and_time(date,time)
     runname=date(3:8)//'_'//time(1:4)
  endif

  if(ismean) then
     ncfnm=trim(outdir)//'/qm_'//trim(runname)//'.nc'
  else
     ncfnm=trim(outdir)//'/qi_'//trim(runname)//'.nc'
  endif

  INQUIRE (file=trim(ncfnm),exist=exans)
  IF (exans) THEN 
    PRINT *,'Clobbering netcdf file: ',trim(ncfnm)
    iret = nf_create(trim(ncfnm),NF_CLOBBER,ncid)
  ELSE
    PRINT *,'Nc_init: Creating netcdf file: ',trim(ncfnm)
    iret = nf_create(trim(ncfnm),NF_NOCLOBBER,ncid)
    if (iret/=0) call print_error('nc_init',iret)
 END IF
 !
 ! specify global attributes
 !
  i=lnblnk(title)
  iret = nf_put_att_text(ncid,NF_GLOBAL,'title',i,title)
  if (ismean) then
     iret = nf_put_att_text(ncid,NF_GLOBAL,'Kind',11,'Mean values')
  else
     iret = nf_put_att_text(ncid,NF_GLOBAL,'Kind',20,'Instantaneous values')
  endif
  i=lnblnk(bnddir)
  iret = nf_put_att_text(ncid,NF_GLOBAL,'Boundary_data_directory',i,bnddir)
  i=lnblnk(outdir)
  iret = nf_put_att_text(ncid,NF_GLOBAL,'Output_directory',i,outdir)
  iret = nf_put_att_int(ncid,NF_GLOBAL,'Landon',NF_INT,1,landon)
  i=lnblnk(SSTmode)
  iret = nf_put_att_text(ncid,NF_GLOBAL,'SSTmode',i,SSTmode)
  iret = nf_put_att_int (ncid,NF_GLOBAL,'Mrestart',NF_INT,1,mrestart)
  iret = nf_put_att_real(ncid, NF_GLOBAL, 'dt',NF_FLOAT,1,dt)
  iret = nf_put_att_real(ncid, NF_GLOBAL, 'viscx',NF_FLOAT,1,viscx)
  iret = nf_put_att_real(ncid, NF_GLOBAL, 'viscy',NF_FLOAT,1,viscy)
  iret = nf_put_att_real(ncid, NF_GLOBAL, 'viscxq',NF_FLOAT,1,viscxq)
  iret = nf_put_att_real(ncid, NF_GLOBAL, 'viscyq',NF_FLOAT,1,viscyq)
  CALL date_and_time(date)
  iret = nf_put_att_text(ncid, NF_GLOBAL, 'History',30, &
      'Created on '//date(1:4)//'-'//date(5:6)//'-'//date(7:8)//' '//&
      time(1:2)//':'//time(3:4)//':'//time(5:6))
  iret = nf_put_att_text(ncid, NF_GLOBAL, 'Conventions',6,'COARDS')
  iret = nf_put_att_text(ncid, NF_GLOBAL, 'Source',9,'QTCM1V2.2')
  iret = nf_put_att_real(ncid, NF_GLOBAL, '_FillValue',NF_FLOAT,1,-9999.)
 !
 ! define different dimensions
 ! 
  iret = nf_def_dim(ncid, 'lon', nlon, dim_lon)
  iret = nf_def_dim(ncid, 'lat', nlat, dim_lat)
  iret = nf_def_dim(ncid, 'time', NF_UNLIMITED, dim_tim)
!
! defining one-dimensional grid variables
!
  iret = nf_def_var(ncid,'lon', NF_FLOAT, 1, dim_lon, id)
  iret = nf_put_att_text(ncid, id, 'long_name',9, 'longitude')
  iret = nf_put_att_text(ncid, id, 'units', 12, 'degrees_east')

  iret = nf_def_var(ncid,'lat', NF_FLOAT, 1, dim_lat, id)
  iret = nf_put_att_text(ncid, id, 'long_name', 8, 'latitude')
  iret = nf_put_att_text(ncid, id, 'units', 13, 'degrees_north')

  iret = nf_def_var(ncid,'time', NF_INT, 1, dim_tim, id)
  if (ismean) then
     iret = nf_put_att_text(ncid,id,'long_name',43,'Model time (last day of averaging interval)')
  else
     iret = nf_put_att_text(ncid,id,'long_name',10,'Model time')
  endif
  write(tstring,'(a,i4,a,i2.2,a,i2.2)') 'days since ', year0,'-',month0,'-',day0
  iret = nf_put_att_text(ncid, id, 'units', 21, tstring)

!
! define 3D (lon,lat,time) fields
!
  dims(1) = dim_lon
  dims(2) = dim_lat
  dims(3) = dim_tim

  
  call nc_defvar('u0     Barotropic zonal velocity   [m/s]')
  call nc_defvar('v0     Barotropic meridional velocity [m/s]')
  call nc_defvar('u1     Mode 1 zonal velocity       [m/s]')
  call nc_defvar('v1     Mode 1 meridional velocity  [m/s]')
  call nc_defvar('T1     Mode 1 temperature          [K]')
  call nc_defvar('q1     Mode 1 humidity             [K]')
  call nc_defvar('vort0  Barotropic vorticity        [1/s]')
  call nc_defvar('psi0   Barotropic stream function  [m^2/s]')
  call nc_defvar('Prec   Precipitation               [W/m^2]')
  call nc_defvar('cl3    Cloud Amount                [-]')
  call nc_defvar('Ts     Surface temperature         [K]')
  call nc_defvar('S0     Incoming Solar Radiation    [W/m^2]')
  call nc_defvar('FSWds  Downward Surface SW Flux    [W/m^2]')
  call nc_defvar('FSWus  Upward Surface SW Flux      [W/m^2]')
  call nc_defvar('FSWut  Upward SW Flux at Top       [W/m^2]')
  call nc_defvar('FLWds  Downward Surface LW Flux    [W/m^2]')
  call nc_defvar('FLWus  Upward Surface LW Flux      [W/m^2]')
  call nc_defvar('OLR    Upward LW Flux at Top       [W/m^2]')
  call nc_defvar('FTs    Surface Sensible Heat Flux  [W/m^2]')
  call nc_defvar('Evap   Surface Latent Heat Flux    [W/m^2]')
  call nc_defvar('Runf   Run-off                     [W/m^2]')
  call nc_defvar('WD     Equivalent water depth      [kg/m^2]')
  call nc_defvar('taux   Zonal Surface Stress        [N/m^2]')
  call nc_defvar('tauy   Meridional Surface Stress   [N/m^2]')
  call nc_defvar('advT1  -T1 Advection               [m/s^2]')
  call nc_defvar('advq1  -Q1 Advection               [m/s^2]')
  call nc_defvar('GMq1   Gross moist stratification  [m/s^2]')
  call nc_defvar('Runs   Surface runoff              [W/m^2]')
  call nc_defvar('Evapi  Interception loss           [W/m^2]')
  call nc_defvar('wet    WD/WD0 Relative wetness     [-]')
!  call nc_defvar('arr1   user defined diagnostic 1   [?]')
  call nc_defvar('arr2   user defined diagnostic 2   [?]')
!  call nc_defvar('arr3   user defined diagnostic 3   [?]')
!  call nc_defvar('arr4   user defined diagnostic 4   [?]')


!
! define 2D (lon,lat) fields
!
  dim_stype(1) = dim_lon
  dim_stype(2) = dim_lat
  iret=nf_def_var(ncid,'STYPE',NF_INT2,2,dim_stype,id)
  iret=nf_put_att_text(ncid,id,'long_name',18,'  Surface Type [-]')
  iret=nf_put_att_text(ncid,id,'units',1,'-')

  iret = nf_enddef(ncid)
  if (iret/=0) call print_error('nc_init',iret)

  do i=1,nlon
    scr(i) = 0. + 360.*real(i-1)/real(nlon)
  end do
  iret = nf_inq_varid(ncid,'lon',id)
  if (iret/=0) call print_error('nc_init',iret)
  iret = nf_put_var_real(ncid,id,scr(1:nlon))  
  if (iret/=0) call print_error('nc_init',iret)

  do i=1,nlat
    scr(i) = -58.125 + 120.*real(i-1)/real(nlat)
  end do
  iret = nf_inq_varid(ncid,'lat',id)
  iret = nf_put_var_real(ncid,id,scr(1:nlat))  

  

  iret = nf_close (ncid)
  if (iret/=0) call print_error('nc_init',iret)
  RETURN
END SUBROUTINE nc_init


SUBROUTINE nc_write_stype(ncfnm,stype,nx,ny)

! write stype to netcdf file as integer map
  IMPLICIT NONE
#include <netcdf.inc>

  INTEGER, INTENT (IN) ::  nx,ny
!  REAL (KIND=8), INTENT(IN) :: stype(nx,ny)
  REAL, INTENT(IN) :: stype(nx,ny)
  INTEGER :: start(2),count(2),iret,ncid,id,i,j
  INTEGER (KIND=2) :: istype(nx,ny)
  CHARACTER (LEN=80), INTENT (IN) :: ncfnm

  start     = (/1,1/)
  count     = (/nx,ny/)

  iret = nf_open (trim(ncfnm), NF_WRITE, ncid)
  iret = nf_inq_varid(ncid,'STYPE',id)
  do j=1,NY
     do i=1,NX
        istype(i,j)=stype(i,j)
     enddo
  enddo
  iret = nf_put_var_int2(ncid,id,istype) 
  iret = nf_close (ncid)

  RETURN
END SUBROUTINE nc_write_stype

  

SUBROUTINE nc_write(ncfnm,varnm,dayofmodel)

  IMPLICIT NONE
!  INCLUDE 'netcdf.inc'
#include <netcdf.inc>
#include "hgrid.h"
 
  INTEGER, INTENT (IN), OPTIONAL  :: dayofmodel
  CHARACTER (LEN=80), INTENT (IN) :: ncfnm
  CHARACTER (LEN=8), INTENT (IN)  :: varnm

  INTEGER, SAVE :: ncall=0
  INTEGER       :: iret,ncid,id,start(3),count(3)
  real          :: rmp,scr(NX,NY)

  iret = nf_open (trim(ncfnm), NF_WRITE, ncid)
 !
 ! write new time level and increment time index
 !
  if (trim(varnm) == 'time') then
    ncall=ncall+1
    iret = nf_inq_varid(ncid,trim(varnm),id)
    iret = nf_put_var1_int(ncid,id,ncall,dayofmodel-1)
 !
 ! write data and zero data arrays
 !
  elseif (ncall >= 1)  then
    iret = nf_inq_varid(ncid,trim(varnm),id)
    if (iret /= 0) then
       print*,'ncwrite: error finding write variable: "',trim(varnm),'"'
       stop
    endif
    start     = (/1,1,ncall/)
    count     = (/NX,NY,1/)

    call geto(scr,varnm)

    iret = nf_put_vara_real(ncid,id,start,count,scr)
    
    if (iret /= 0) stop 'nc_write: error writing variable'
 !
 ! error control
 !
  else
    if (ncall < 1)      stop 'nc_write: indexing error'
  end if
  iret = nf_close (ncid)

  RETURN
END SUBROUTINE nc_write


SUBROUTINE nc_write_i(ncfni,varni,var,dayofmodel)

  IMPLICIT NONE
!  INCLUDE 'netcdf.inc'
#include <netcdf.inc>
#include "hgrid.h"
 
  INTEGER, INTENT (IN), OPTIONAL  :: dayofmodel
  CHARACTER (LEN=80), INTENT (IN) :: ncfni
  CHARACTER (LEN=8), INTENT (IN)  :: varni
  REAL, INTENT (IN)               :: var(NX,NY)

  INTEGER, SAVE :: ncall=0
  INTEGER       :: iret,ncid,id,start(3),count(3)
  real          :: rmp

  iret = nf_open (trim(ncfni), NF_WRITE, ncid)
 !
 ! write new time level and increment time index
 !
  if (trim(varni) == 'time') then
    ncall=ncall+1
    iret = nf_inq_varid(ncid,trim(varni),id)
    iret = nf_put_var1_int(ncid,id,ncall,dayofmodel-1)
 !
 ! write data and zero data arrays
 !
  else
    iret = nf_inq_varid(ncid,trim(varni),id)
    if (iret /= 0) then
       print*,'ncwrite_i: error finding write variable: "',trim(varni),'"'
       print*,'in netcdf file "',ncfni,'"'
       stop
    endif
    start     = (/1,1,ncall/)
    count     = (/NX,NY,1/)
    iret = nf_put_vara_real(ncid,id,start,count,var)  
    if (iret /= 0) then
       print*, 'nc_write_i: error writing variable id = ',id
       stop
    end if
  end if
  iret = nf_close (ncid)

  RETURN
END SUBROUTINE nc_write_i


SUBROUTINE nc_defvar(longvarname)

  USE nc_file_data

  implicit none
#include <netcdf.inc>
  
  character (len=*), intent(in) :: longvarname

  integer :: id,i1,i2,len,lnblnk,iret

! use first word in longvarname as variable name
  i1=index(longvarname,' ')
  if(i1.le.1) i1=lnblnk(longvarname)
  if(i1.ge.8) then
    print*,'nc_init: Error variable name too long!'
    stop
 endif

  iret=nf_def_var(ncid,longvarname(1:i1-1),NF_FLOAT,3,dims,id)
  if (iret/=0) call print_error('nc_defavr',iret)
!  print*,'nc_defvar: Defined var "',longvarname(1:i1-1),'"',' id = ',id

  i2=lnblnk(longvarname)
  iret=nf_put_att_text(ncid,id,'long_name',i2-i1,longvarname(i1+1:i2))
  if (iret/=0) call print_error('nc_defavr',iret)

! get units string out of brakets in longvarname
  i1=index(longvarname,'[')+1
  i2=index(longvarname,']')-1
  len=i2-i1+1
  if(len.gt.1) then
     iret=nf_put_att_text(ncid,id,'units',len,longvarname(i1:i2))
  else
     iret=nf_put_att_text(ncid,id,'units',7,'unknown')
  endif
  if (iret/=0) call print_error('nc_defavr',iret)

  return
END SUBROUTINE nc_defvar


SUBROUTINE print_error(routine,iret)
  IMPLICIT NONE
  INCLUDE 'netcdf.inc'

  INTEGER, INTENT (IN) :: iret
  INTEGER              :: lnblnk
  CHARACTER (len=*)    :: routine
  CHARACTER (len=80)   :: error_string

  print*,'NetCDF:  netcdf write error: no.',iret
  error_string=nf_strerror(iret)
  print*,routine,': "',error_string(1:lnblnk(error_string)),'"'
  stop routine
END SUBROUTINE print_error
