c*********************************************************************
c $Source: /long/munnich/CVSROOT/UCLA/QTCM/src/output.F,v $
c $Author: munnich $
c $Revision: 1.5 $
c $Date: 2000/08/03 18:27:53 $
c
c File: output.f
c
c This file contains the output subroutines for qtcm model
c
c Code history:
c   Original version:  Ning Zeng (Jan 1997)
c   Revised by Ning Zeng (May 1997):  Separate sum and output for pmean 
c   Heavy revision by Ning Zeng (May 2000):  
c     using a version of pavr.f tailored specifically to output mean 
c     variables. This is simpler and a bit faster than directly using pavr.f
c     because the user has to type variable names twice in using pavr.f.
c     As a tradeoff, this routine [outpmean] is less flexible than pavr.f
c     so that all the variables are 'bound' together in terms of output
c     frequency and destination. If more flexible user control is needed, use
c     pavr.f (very easy to use).
c
c User control on output:
c   1. To skip variable for output:
c      Simply comment out the corresponding call to [winst] in [outpinst]
c      for instantenous values and/or [oacc] in [outpmean] for mean values 
c      such as 
c         call winst(u1)       !output instantenous values of u1
c         call oacc(u1)        !accumulate values of u1 (output automatic)
c      one can also comment out in bulk the
c      call to [outpinst]/[outpmean] in [outpall] so not to output at all;
c      one can also control at an even higher level in the main program
c      [driver], or by changing the values of <ntouti>,<ntout>(see following)
c   2. As in 1, but add; 
c   3. To control how often and where to output a variable:
c      By default, output is done every <ntout> days to the three files
c      described above, and the first <noout> days are not output at all.  
c      <ntout> can also be specified as a user-
c      control parameter in driver.in (via *.sh), as well as <ntouti>,
c      which controls the instantaneous value output.
c      Note: The minimum value for <ntout> and <ntouti> is 1 day, so if you 
c      need to output
c      more often than a day, you need to call the output routines from with-
c      in the atmosphere-land coupling loop in [qtcm]
c
c Usage note:
c   for output means, make sure to call [varmean] from the innermost loop of
c     the model (in the atmo-land coupling loop in [qtcm]).
c
c The following header files are used using INCLUDE:
c   qtcm.h       Primary subroutine common blocks and parameter of qtcm
c   land.h       
c   calendar.h   Variable declarations for virtual calendar
c   outpmean.h      For output mean values
c
c File I/O:  there are files that remain open throughout (global), others
c   are closed immediately in the corresponding opening routine before
c   exit (local).
c   Convention: units 20-29 are reserved for global files (always output);
c   units 10-19 for local files (usually input except for qtcm.restart.out);
c   no other units are used.
c
c Output the following (see [outpall]):
c   1. Instantaneous value of selected fields       unit=21,file=q.out
c   2. Averaged values of selected fields           unit=22,file=qmean.out
c   3. Time series of a few selected variables      unit=27,file=timeseries
c   4. output for restart                    in qtcm.f;file=qtcm.restart.out
c*********************************************************************
c

             subroutine outpinit
c************
c Open the global output files
c
      implicit none
      include 'driver.h'

! Functions
      integer lnblnk

c --File I/O:  The following units are opened by this routine
c   and remain open on exit from the routine; these are the
c   only global files:
c     unit 21 = output file q.out
c     unit 22 = output file qmean.out
c     unit 27 = output file timeseries.out
c
c --Local Variables and Functions:
c*****************************************************************

c open the global output files
      open(21,file=outdir(1:lnblnk(outdir))//'/q.out',
     &     form='unformatted',status='unknown')
      open(22,file=outdir(1:lnblnk(outdir))//'/qmean.out',
     &     form='unformatted',status='unknown')
      open(27,file=outdir(1:lnblnk(outdir))//'/timeseries.out',
     &     form='formatted',status='unknown')

      return
      end


             subroutine outpall
c************
c Output the following:
c 1.  instantaneous values of selected fields    unit=21,file=q.out
c 2.  averaged values of selected fields        unit=22,file=qmean.out
c 3.  time series of a few variables            unit=27,file=timeseries
c 4.  output for restart                        in qtcm.f;file=qtcm.restart.out
c
      implicit none
#include "calendar.h"
      include 'hgrid.h'
      include 'driver.h'
      include 'qtcm.h'
      include 'land.h'


c- File I/O:  Unit 21,22,27 remain open

c- Local variables

c*****************************************************************


c== output instanteneous values every <ntouti> days
c Note: if need to call within a day, call from within boundary-atmo coupling 
c       loop in [qtcm]

      call outpinst

c== output mean values every <ntout> days

      call outpmean

c== dirty and quick output for diagnosing model run at runtime
      write(27,*)T1(NX/2+1,(NY+1)/2),u1(NX/2+1,(NY+1)/2)
     1           ,Ts(NX/2+1,(NY+1)/2),Ts(55,15),Qc(55,15)


c== Output all the prognostic variables for restart run every 12*ntout
c   time step in case the model is abnormally interupted
      if(mod(dayofmodel,12*ntout).eq.0) call out_restart 

      return
      end

             subroutine outpinst
c************
c Output the following:
c 1.  instantaneous value of selected fields    unit=21,file=q.out
c
      implicit none
      include 'hgrid.h'
#include "calendar.h"
      include 'driver.h'
      include 'qtcm.h'
      include 'land.h'

c- File I/O:  Unit 21,22,27 remain open

c- Local function
      logical outtime

c*****************************************************************

c== output instanteneous values every <ntouti> days
c if need to call within a day, call from within boundary-atmo coupling 
c   loop in [qtcm]

      if(outtime(ntouti,noout)) then
      call winst(u1)    
      call winst(v1)    
      call winst(T1)    
      call winst(q1)    
      call winst(u0)    
      call winst(v0)    
      call winst(vort0)    
      call winst(psi0)    
      call winst(Ts)    
      call winst(Qc)    
      call winst(cl3)    
      call winst(S0)    
      call winst(FSWds)    
      call winst(FSWus)    
      call winst(FSWut)    
      call winst(FLWds)    
      call winst(FLWus)    
      call winst(FLWut)    
      call winst(FTs)    
      call winst(Evap)    
      call winst(Runf)    
      call winst(WD)    
      call winst(STYPE)    
      call winst(taux)    
      call winst(tauy)    
      call winst(advT1)    
      call winst(advq1)    
      call winst(arr1)    
      call winst(arr2)    
      call winst(arr3)    
      call winst(arr4)    
      endif

      return
      end

             subroutine winst(p)
c************
c Output the following:
c 1.  instantaneous value of selected fields    unit=21,file=q.out
c- File I/O:  Unit 21,22,27 remain open
c************
      implicit none
      include 'hgrid.h'
      real p(*)
      integer k

      write(21)  (p(k),k=1,NXY)   !write to file

      return
      end


             subroutine outpmean
c************
c Output the following:
c 2.  averaged values of selected fields        unit=22,file=qmean.out
c
      implicit none
      include 'hgrid.h'
#include "calendar.h"
      include 'driver.h'
      include 'outpmean.h'

c- File I/O:  Unit 21,22,27 remain open

c- Local variables
      integer k,iv
      real rpsum,pmean(NXY)

c- logical function
      logical outtime

c*****************************************************************
c== output mean values every <ntout> days or at end of month

      if(outtime(ntout,noout)) then
        rpsum = 1./npsum
        do iv=1,nvar         !do this for all the averaged variables
                                  !  which are specified in [varmean]
          do k=1,NXY       
            pmean(k) = psum(k,iv) * rpsum
            psum(k,iv) = 0.       !averaging; reset the counter and the array
          end do

          write(22)  (pmean(k),k=1,NXY)       !write to file

        end do
        npsum = 0
      endif

      return
      end

             subroutine varmean(it)
c************
c accumulate for the variables that need averaging
c
      implicit none
      include 'hgrid.h'
#include "calendar.h"
      include 'qtcm.h'
      include 'land.h'
      include 'outpmean.h'

      integer it

c --File I/O:  Unit 21,22,27 remain open
c*****************************************************************

      npsum=npsum+1            !# of sumups
      nvar=0              !reset for this round of summing

      call oacc(u1)    
      call oacc(v1)    
      call oacc(T1)    
      call oacc(q1)    
      call oacc(u0)    
      call oacc(v0)    
      call oacc(vort0)    
      call oacc(psi0)    
      call oacc(Ts)    
      call oacc(Qc)    
      call oacc(cl3)    
      call oacc(S0)    
      call oacc(FSWds)    
      call oacc(FSWus)    
      call oacc(FSWut)    
      call oacc(FLWds)    
      call oacc(FLWus)    
      call oacc(FLWut)    
      call oacc(FTs)    
      call oacc(Evap)    
      call oacc(Runf)    
      call oacc(WD)    
      call oacc(STYPE)    
      call oacc(taux)    
      call oacc(tauy)    
      call oacc(advT1)    
      call oacc(advq1)    
      call oacc(arr1)    
      call oacc(arr2)    
      call oacc(arr3)    
      call oacc(arr4)    

      if(nvar.gt.NVARMAX) 
     &  stop 'terror !! NVARMAX too small in outpmean.h'

      return
      end


      subroutine oacc(p)
c*********************
c Accumulate (sum up) for variable p
c
c******************************************************************
      implicit none
      include 'hgrid.h'
      include 'outpmean.h'

c- arguments
      real p(NXY)

      nvar=nvar+1   !the current variable is #var-index in psum

c== sum up
      call dosum(p,psum(1,nvar))

      return
      end


      subroutine dosum(p,psum)
c add array p to psum
      implicit none
      include 'hgrid.h'
      real p(NXY), psum(NXY)
      psum=psum+p  ! array
      return
      end


      logical function outtime(ntout,noout)
!************
!     --- decide if the model time is an output time

      implicit none
   
#include "calendar.h"
      integer  ntout,noout

      logical outtime

      outtime=.false.   

      if(dayofmodel.le.noout) return  !no output the first noout days

      if(ntout.eq.-30) then     ! output at end of month
         outtime=dayofmonth.eq.monlen(monthofyear) 
      else                      ! outpmean every <ntout> days
         outtime=mod(dayofmodel-noout,ntout).eq.0
      endif

      return
      end
 
