c $Source: /long/munnich/CVSROOT/UCLA/QTCM/src/qtcm.h,v $
c $Author: munnich $
c $Revision: 1.7 $
c $Date: 2000/08/03 18:27:54 $
c
c Header file for qtcm; qtcm variables and parameters
c
      integer NEQ,NEQ0,NEQ2,NSTYPE,NVMOD,NTMOD,NZ
      parameter(NEQ=4)      !baroclinic mode: u,v,T,q
      parameter(NEQ0=4)     !barotropic mode:u0,v0,vort0,psi0
      parameter(NEQ2=2)     !boundary layer mode: u2,v2 not used this version
      parameter(NSTYPE=4)   !surface type; ocean or vegetation type over land
      parameter(NVMOD=2)    !vertical velocity modes; needed for advection
      parameter(NTMOD=1)    !vertical temperature profile modes
      parameter(NZ=10)      !number of vertical levels in z coordinate


c- Model variables
c baroclinic mode
      REAL u1(NX,NY),v1(NX,0:NY),T1(NX,NY),q1(NX,NY),div1(NX,NY)
     1      ,advu1(NX,NY),advv1(NX,0:NY),advT1(NX,NY),advq1(NX,NY)
     2      ,dfsu1(NX,NY),dfsv1(NX,0:NY),dfsT1(NX,NY),dfsq1(NX,NY)
     3      ,GMq1(NX,NY)
      common/barclcom/ u1,v1,T1,q1,div1,advu1,advv1,advT1,advq1
     1      ,dfsu1,dfsv1,dfsT1,dfsq1,GMq1

c model physics variables associated with temperature mode 1, including:
c   convection, radiation, cloud, heat and moisture fluxes
      REAL Qc(NX,NY),Hc(NX,NY)             !prec.; convection criterion
     1       ,FLWds(NX,NY),FLWus(NX,NY)      !down/up LW fluxes at surface
     2       ,FLWut(NX,NY),FLW(NX,NY)        !LW at top; LW column total
     3       ,S0(NX,NY)                      !incoming solar at top
     4       ,FSWds(NX,NY),FSWus(NX,NY)      !down/up SW fluxes at surface
     5       ,FSWut(NX,NY),FSW(NX,NY)        !SW at top; SW absorbed by atmo.
     6       ,cl3(NX,NY)                     !cloud cover
     7       ,Evap(NX,NY),FTs(NX,NY)         !evap and sensible heat
     8       ,taux(NX,NY),tauy(NX,NY)        !surface stress
     9       , co2(NX,NY)                    !carbon dioxide
      common/phys1com/ Qc,Hc,FLWds,FLWus,FLWut,FLW
     1                ,S0,FSWds,FSWus,FSWut,FSW,cl3
     2                ,Evap,FTs,taux,tauy,co2

c barotropic mode V0; note only vort0 and u0bar are prognostic
      REAL u0bar,u0(NX,NY),v0(NX,0:NY),vort0(NX,NY),psi0(NX,0:NY)
     1      ,div0(NX,NY),chi0(NX,NY)
     2      ,advu0(NX,NY),advv0(NX,0:NY),dfsu0(NX,NY),dfsv0(NX,0:NY)
     3      ,rhsu0bar(3),rhsvort0(NX,NY,3)
      common/bartrcom/u0bar,u0,v0,vort0,psi0,div0,chi0,advu0,advv0
     1      ,dfsu0,dfsv0,rhsu0bar,rhsvort0

c ekman   @not used in this version
      REAL u2(NX,NY),v2(NX,NY),div2(NX,NY)
     1    ,rhsu2(NX,NY,3),rhsv2(NX,NY,3)
      REAL ruRe(NX,NY),ruIm(NX,NY) ! regression coeff. to get surface winds
      common/ekmancom/u2,v2,div2,rhsu2,rhsv2,ruRe,ruIm

c temporary arrays
      REAL arr1(NX,NY),arr2(NX,NY),arr3(NX,NY),arr4(NX,NY)
      common/arrcom/ arr1,arr2,arr3,arr4


      
c- Boundary inputs and derived; exception: Ts overland is prognostic
      REAL Ts(NX,NY),Ts0(NX,NY),STYPE(NX,NY)          !STYPE real for output
      REAL CDN(NX,NY),Vs(NX,NY),CV(NX,NY),ALBDs(NX,NY),TOP(NX,NY)
      common/bndrycom/ Ts,Ts0,CDN,Vs,CV,ALBDs,TOP,STYPE


c- Physical constants, model parameters

c physical/math constants
      REAL Rearth,rhoair,Rair,Omega,Hlatent,Cp        !physical constants
     1       ,zero,one,two,half,pi                      !math constants
      REAL dx,dy,fu(NY),fv(0:NY)                      !grid size, Coriolis
      REAL cosu(NY),cosv(0:NY),cosui(NY),cosvi(0:NY)  !cos(latitude)

c indexing for finite differencing at the periodic x boundary
      integer im1(NX),ip1(NX)
c factors to handle finite differencing in advection at y boundaries
      REAL rj1(NY),rj(NY),rj_1(NY)
c factors to handle finite differencing in diffusion at y boundaries
      REAL dj1(NY),dj(NY),dj_1(NY)


c sponge boundary for heat flux, diffusion, momentum 
      REAL spngh0(NY),spngh1(NY),spngh2(NY),spngh3(NY)    !sponge boundary
      REAL spngh4(NY)

c model physics related parameters; read from par.in generated by par.f
      REAL a1hat,a1s,V1s,V2s,b1hat,b1s,bb1hat,V1z(NZ) !T,q profiles
     1       ,GMsr,GMsp,GMqr,GMqp,GMs0r,GMq0r,GMs2r,GMs2p,GMq2r,GMq2p !stability
     2       ,eps_c                                     !Betts-Miller
     3       ,CV0                                       !drag
     4       ,eps_0,eps_10,eps_20,eps_01,eps_1,eps_21   !momentum damping
     5       ,eps_02,eps_12,eps_2
     2       ,Tsref,Cpg                                 !surface
     3       ,Trefs,qrefs                               !T,q ref. at surface
     3       ,Tcrefs,qcrefs                             !Tc,qc ref. at surface
     3       ,Trefhat,qrefhat                           !T,q ref. hat 
     3       ,Tcrefhat,qcrefhat                         !Tc,qc ref. hat 
     4       ,Vijk(0:NVMOD-1,0:NVMOD-1,0:NVMOD-1)       !advection of V
     4       ,Vwijk(0:NVMOD-1,0:NVMOD-1,0:NVMOD-1)      !vertical advection of V
     5       ,DTijk(0:NVMOD-1,NTMOD,NTMOD)              !advection of T
     6       ,Dqijk(0:NVMOD-1,NTMOD,NTMOD)              !advection of q
      common/parcom/Rearth,rhoair,Rair,Omega,Hlatent,Cp
     1       ,zero,one,two,half,pi
     2       ,dx,dy,fu,fv,cosu,cosv,cosui,cosvi
     3       ,im1,ip1,rj1,rj,rj_1,dj1,dj,dj_1
     3       ,spngh0,spngh1,spngh2,spngh3,spngh4
     4       ,a1hat,a1s,V1s,V2s,b1hat,b1s,bb1hat,V1z
     5       ,GMsr,GMsp,GMqr,GMqp,GMs0r,GMq0r,GMs2r,GMs2p,GMq2r,GMq2p
     6       ,eps_c
     7       ,CV0
     8       ,eps_0,eps_10,eps_20,eps_01,eps_1,eps_21
     9       ,eps_02,eps_12,eps_2
     6       ,Tsref,Cpg
     7       ,Trefs,qrefs
     8       ,Tcrefs,qcrefs,Trefhat,qrefhat,Tcrefhat,qcrefhat
     8       ,Vijk,Vwijk,DTijk,Dqijk

