c*********************************************************************
c $Source: /long/munnich/CVSROOT/UCLA/QTCM/src/rdsst.F,v $
c $Author: munnich $
c $Revision: 1.7 $
c $Date: 2000/06/27 01:01:41 $
c
c File: ocean.f
c
c Read SST data files and interpolate
c
c The date information is converted into SST file names:
c   In this version, bc file names include the date and variable name
c   in this way:
c
c               file name = yyyymmdd.var
c
c   where yyyy = year , mm = month, dd=dayofmonth, var = a variable name.
c
c   if SSTmode is seasonal, yyyy=0000
c
c Code history:
c   Original version:  Bill Weibel  Nov 1996
c   Partial revision:  Ning Zeng    Jan 1997
c
c Routines called but not in this file
c   julian             in driver.f
c*********************************************************************


      Subroutine OceanInit()
c
c  Get the initial SST data for the dates on either side of an
c  arbitrary starting date
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "rdsst.h"
#include "calendar.h"

      integer i,j
 
      write (*,*) "OceanInit: get the initial SST data"

c-  get the dates(centered on the 15th) on either side of the starting date.
c before the 15th
      if(day0.ge.1.and.day0.le.14) then 
        date1 = year0*10000 + (month0-1)*100 + 15
        if(month0.eq.1)date1= (year0-1)*10000 + 12*100 + 15
        date2= year0*10000 + month0*100 + 15
        if (SSTmode.eq.'real_time')  then
      write(*,*)'Reminder: your input requires SST data before the'
      write(*,*)' model starting date! if non-existant, model will stop'
      write(*,*)' Remedy: change the starting date or get the data'
        end if
      end if
c after the 15th
      if(day0.ge.15.and.day0.le.31) then
        date1= year0*10000 + month0*100 + 15
        date2 = year0*10000 + (month0+1)*100 + 15
        if(month0.eq.12)date2= (year0+1)*10000 + 1*100 + 15
      end if
      
      call sstin(date1, Tprior)

      do j=1,NY
      do i=1,NX
        Tnow(i,j) = Tprior(i,j)               !Fudge the initial T
      end do
      end do

      if (SSTmode .eq. 'perpetual') return

      call sstin(date2, Tnext)

      return
      end
      
      
      Subroutine Ocean(ndays,it)
      implicit none
#include "hgrid.h"
#include "calendar.h"
#include "driver.h"
#include "rdsst.h"
      logical TimetoReadSst
      integer julian
      integer ndays
      integer month,day1,day2,it
      REAL thistime

      integer i,j

      if (SSTmode .eq. 'perpetual') return

      if (TimetoReadSst(date1, date2, dateofmodel,SSTmode)) then
c        Tprior = Tnext
         do j=1,NY
         do i=1,NX
           Tprior(i,j) = Tnext(i,j)
         end do
         end do
         date1 = date2
         date2 = date1 + 100                     !advance date2 by 1 month
         month=mod(date1-15,10000)/100           !if month=12
         if(month.eq.12)date2= date1+10000-1100   !  set to Jan of next year
         call sstin(date2, Tnext)
      endif
c
c     timeinterp requires julian dates or some other monotonic time.
      day1 = julian(date1)
      day2 = julian(date2)
      thistime = dayofmodel + ndays/2.       ! compute timemean of sst
      call TimeInterp(Tprior,Tnext,Tnow ,day1,day2,thistime,nx,ny)
c     write (*,*) 'Ocean: interpolated to t=',thistime
c    $            ,'date1,day1,date2,day2=',date1,day1,date2,day2
      return
      end
      
      
      logical function TimetoReadSST(date1,date2,now,SSTmode)
      integer date1, date2, now
      character*(*) SSTmode
      TimetoReadSST = now .ge. date2
      return
      end


      Subroutine sstin(date, Tdummy)
c
c   Return:
c     date  -   not changed
c     Tdummy  -   the new temperatures.
c
c
c   In this version, bc file names include the date and variable name
c   in this way:
c
c		file name = yyyymmdd.var
c
c   where yyyy = year , mm = month, dd=dayofmonth, var = a variable name.
c
c   if SSTmode is seasonal, yyyy=0000
c
      implicit none
#include "hgrid.h"
#include "driver.h"
#include "rdsst.h"
#include "calendar.h"
      REAL Tdummy(nx,ny)
      integer date
      character*(8) SSTfile
! Functions
      integer lnblnk

      write(SSTfile,'(i8.8)') date         !convert the date into a string

c get the climatological SST for seasonal mode.
      if (SSTmode.eq.'seasonal')  then
        SSTfile='0000'//SSTfile(5:8)   !switch the year to 0000
      end if

c get the perpetual SST; the sst file must be 00000000.sst
      if (SSTmode.eq.'perpetual')  then
        SSTfile='00000000'
      end if

c read the SST
      open(13,file=SSTdir(1:lnblnk(SSTdir))//'/'//SSTfile//'.sst'
     *       ,form='formatted',status='old')
      read(13,*) Tdummy                                  !in Celsius
      close(13)

 
      write(*,*) 'sstin: Read SST at model date', dateofmodel,
     *            ' from file'
      write(*,*)'        "',
     *      SSTdir(1:lnblnk(SSTdir))//'/'//SSTfile//'.sst','"'

      return
      end


      Subroutine getSST(T_out)
c************
c  A method for returning the Sea Surface Temperature to other models
c  In this case, it simply provides a pointer to the data
c  structure of the ocean.
c
      implicit none
#include "hgrid.h"
#include "rdsst.h"
      REAL T_out(nx,ny)
c************************************************************
 
      integer i,j
 
      do j=1,NY
      do i=1,NX
        T_out(i,j) = Tnow(i,j)
      end do
      end do
 
      return
      end

c************
