#!/bin/csh -f                         
# a run with options for coupled ocean-atmosphere(QTCM with mixed-layer model)

# Set the QTCM root directory (e.g., where your files are untared)
set QTCMROOT = ..       # this is to assume you are in QTCMROOT/work right now

set procdir = $QTCMROOT/proc     # Set the processing directory

set workdir = $QTCMROOT/work     # Set the working directory

# Set the source directory which contains all the source files
set srcdir = $QTCMROOT/src


#*******
# Step 1 (control run): 
# run qtcm_seasonal run to generate Qflux (net sfc. flux + Ts tendency)
#****************************
#- Compile qtcm using make in srcdir, but run it in workdir
cd $srcdir

# Use qtcm with GrADS binary output for Q-flux computation 
make qtcm_qflx 
cd $workdir

# copy qtcm to work directory
cp -fp ../src/qtcm_qflx qtcm

#
#Generate the user control parameters.
#
sed 's/\!.*$//' > driver.in << EOF
 &driverdata
 title='QTCM seasonal run'      !title
 bnddir='$QTCMROOT/bnddata'      !bnd. data other than SST
 SSTdir='$QTCMROOT/bnddata/SST_Reynolds_oi'    !where SST files are
 outdir='$QTCMROOT/proc'      !where output go
 landon=1                     !if not 1: land is like ocean with fake 'SST'
 SSTmode='seasonal'           !decide what kind of SST to use
 year0 = 1001                 !starting year; must be between 1001 and 9998
 month0 = 11                  !starting month; 1 to 12
 day0 = 1                     !1 t0 31; each month has variable or fixed length
 lastday = 1156               !model runs from day 1 to lastday 
 interval = 1                 !ocean-atmo. coupling interval; integer days
 noout= 61                    !no output for the first noout days
 ntout= -30     !output mean every ntout days (-30: calendar monthly mean)
 ntouti=365     !output inst. value every ntouti days (-30 every calendar month)
 mrestart=1                   !=1: restart using qtcm.restart
 dt=1200.                     !atmospheric model time step, real seconds
 &end
EOF

#- Run qtcm using nice; timing the run; diagnostic output to log
/bin/time nice qtcm |& tee log 

cp -f qtcm.restart.out qtcm.restart
mv -f $QTCMROOT/proc/qmean.out qmean_control.out


#*******
# Step 2 
# use the output generated from control run (Step1) to get 
#   annual mean and seasonal averaged Fsnet and temp tendency
#****************************
#
 make aveflux 
 aveflux
 echo 'Step 2 finished'

#*******
# Step 3 
# Run qtcm using the mixed-layer ocean model
# Note: users must uncomment 'call cplmean' in qtcm.f 
#*************************************************

#- Compile qtcm using make in srcdir, but run it in workdir
cd $srcdir
make qtcm_mxl
cd $workdir

# copy qtcm_mxl to work directory
cp -fp ../src/qtcm_mxl qtcm_mxl

#
#
sed 's/\!.*$//' > driver.in << EOF
 &driverdata
 title='QTCM mixed-layer run'      !title
 bnddir='$QTCMROOT/bnddata'      !bnd. data other than SST
 SSTdir='$QTCMROOT/bnddata/SST_Reynolds_oi'    !where SST files are
 outdir='$QTCMROOT/proc'      !where output go
 landon=1                     !if not 1: land is like ocean with fake 'SST'
 SSTmode='seasonal'           !decide what kind of SST to use
 year0 = 1001                 !starting year; must be between 1001 and 9998
 month0 = 11                  !starting month; 1 to 12
 day0 = 1                     !1 t0 31; each month has variable or fixed length 
 lastday = 3711               !model runs from day 1 to lastday 
 interval = 1                 !ocean-atmo. coupling interval; integer days
 noout= 61                    !no output for the first noout days
 ntout=-30     !output mean every ntout days (-30: calendar monthly mean)
 ntouti=365    !output inst. value every ntouti days (-30 every calendar month) 
 mrestart=1                   !=1: restart using qtcm.resart
 dt=1200.                     !atmospheric model time step, real seconds
 &end
EOF
 
#- Run qtcm using nice; timing the run; diagnostic output to log
/bin/time nice qtcm_mxl |& tee log
