#!/bin/csh -f
# qtcm_realtime.sh   Zeng 5/1998
# a 17yr run with Smith-Reynolds SST from 1982-1998

# Set the QTCM root directory (e.g., where your files are untared)
set QTCMROOT = ..       # this is to assume you are in QTCMROOT/work right now

set workdir = $QTCMROOT/work     # Set the working directory

# Set the source directory which contains all the source files
set srcdir = $QTCMROOT/src

#- Compile qtcm using make in srcdir, but run it in workdir
cd $srcdir
make qtcm
cd $workdir

# copy qtcm to work directory
cp -fp ../src/qtcm qtcm


#
#Generate the user control parameters.
#
sed 's/\!.*$//' > driver.in << EOF
 &driverdata
 title='QTCM 82-98 real time  run'      !title
 bnddir='$QTCMROOT/bnddata'   !bnd. data; but see [qtcm] for seasonal bnd.
 SSTdir='$QTCMROOT/bnddata/SST_Reynolds_oi'    !where SST files are
 outdir='$QTCMROOT/proc'      !where output go
 landon=1                     !if not 1: land is like ocean with fake 'SST'
 SSTmode='real_time'          !decide what kind of SST to use
 year0 = 1981                 !starting year; must be between 1001 and 9998
 month0 = 11                  !starting month; 1 to 12
 day0 = 1                     !1 t0 31; each month has variable or fixed length
 lastday = 6266               !model runs from day 1 to lastday 
 interval = 1                 !ocean-atmo. coupling interval; integer days
 noout=61                     !no output for the first noout days (1981 here)
 ntout=-30      !output mean every ntout days (-30: calendar monthly mean) 
 ntouti=365     !output inst. value every ntouti days (-30 every calendar month)
 mrestart=1                   !=1: restart using qtcm.resart
 dt=1200.                     !atmospheric model time step, real seconds
 &end
EOF

#- Run qtcm using nice; timing the run; diagnostic output to screen and log
/bin/time nice qtcm |& tee log 

