#!/bin/csh -f
#spinup.sh   Zeng/Weibel   Jan 1997 
#  A spin-up run
#  Run this if(otherwise run qtcm_seasonal.sh, etc.):
#    1.  your computer does not read the provided unformatted restart file
#    2.  model physics has been significantly changed.
#  In this example, a seasonal run starts at Jan.1 and ends at Dec.30.
#    no restart file is needed (mrestart=0).
#    This is useful when your computer is not compatible with the (only!)
#    provided binary file qtcm.restart. 
#    More importantly, any significant change in physics may contaminate
#    the first month or two, so this run would give you a relatively
#    balanced initial state given a particular physics.


# Set the QTCM root directory (e.g., where your files are untared)
set QTCMROOT = ..       # this is to assume you are in QTCMROOT/work right now

set workdir = $QTCMROOT/work     # Set the working directory

# Set the source directory which contains all the source files
set srcdir = $QTCMROOT/src

set thedate = `date +%H%M_%m%d`  # current date and time (used for file names)
set theoutdir = $QTCMROOT/proc # output directory

#- Compile qtcm using make in srcdir, but run it in workdir
pushd $srcdir
make qtcm
popd

# copy qtcm to work directory
cp -fp ../src/qtcm qtcm

#
#Generate the user control parameters for a one year run; spin up the
#  atmosphere with land off
#
sed 's/\!.*$//' > driver.in << EOF
 &driverdata
 title='QTCM spin-up run land off'     !title
 landon=0                     !=0: land is like ocean with faked SST
 noout=365                    !no output for the first noout days
 mrestart=0                   !=1: restart using qtcm.resart
 &end
EOF

# bnddir='$QTCMROOT/bnddata'      !bnd. data other than SST
# SSTdir='$QTCMROOT/bnddata/SST_Reynolds_oi'    !where SST files are
# outdir='$QTCMROOT/proc'      !where output go
# SSTmode='seasonal'           !decide what kind of SST to use
# lastday = 365                !model runs from day 1 to lastday 
# interval = 1                 !ocean-atmo. coupling interval; integer days
# year0 = 1001                 !starting year; must be between 1001 and 9998
# month0 = 1                   !starting month; 1 to 12
# day0 = 1                     !1 t0 31; each month has variable or fixed length
# noout=0                      !no output for the first noout days
# ntout=-30     !output mean every ntout days (-30: calendar monthly mean) 
# ntouti=-30    !output inst. value every ntouti days (-30 every calendar month)
#dt=1200.                      !atmospheric model time step, real seconds

#- Run qtcm using nice; timing the run; diagnostic output to log
/usr/bin/time nice qtcm |& tee log 


#
# use the restart file generated from above for next run
#
cp qtcm.restart.out qtcm.restart
cp qtcm.restart.out qtcm.restart.spinup1


#
# spin-up the land with land on using restart
#
sed 's/\!.*$//' > driver.in << EOF
 &driverdata
 title='QTCM 1yr seasonal run land on'      !title
 landon=1                     !=1: land is interactive (default)
 noout=365                           !no output for the first noout days
 mrestart=1                   !=1: restart using qtcm.resart (default)
 &end
EOF

#- Run qtcm using nice; timing the run; diagnostic output to log
/bin/time nice qtcm |& tee log 

#
#  Now a spun-up(pretty much) beginning Jan. 1  condition is ready
#
cp qtcm.restart.out qtcm.restart
